/******************************************************************
 Project Name: Terminal Tetris
 File Name: main.c
 Description:  Լ ִ .
 
 Created: 2004. 12. 22
 RCS-ID: $Id: main.c,v 1.49 2004/06/18 06:28:14 myunggoni Exp $
 Copyright: (c) 2004 by myunggoni
 License: GNU General Public License
 Author: Myung-gon, Park <myunggoni@users.kldp.net>
 *******************************************************************/
 
#include <ncurses.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <getopt.h>
#include "main.h"
#include "config.h"

const char *program_name;

int main(int argc, char *argv[]) {
	int next_option;
	int level = 1;
	int server_mode = 0;
	unsigned short int port;
	const char *const short_options = "hl:sc:p:";
	char *start_level = NULL;
	char *server_address = NULL;
	char *port_number = NULL;
	const struct option long_options[] = {
		{"help", 0, NULL, 'h'},
		{"level", 1, NULL, 'l'},
		{"server", 0, NULL, 's'},
		{"client", 1, NULL, 'c'},
		{"port", 1, NULL, 'p'},
		{NULL, 0, NULL, 0}
	};
	
	
	program_name = argv[0];

	do {
		next_option = getopt_long(argc, argv, short_options, long_options, NULL);

		switch(next_option) {
			case 'h': /* -h or --help */
				print_usage(stdout, 0);
				break;
				
			case 'l': /* -l or --level */
				start_level = optarg;

				if(is_number(start_level)) {
					level = atoi(start_level);

					if(level == 0) {
						fprintf(stderr, "level: invalid start level(input non-zero value).\n",DEFAULT_LEVEL);
						exit(1);
					} else if(level > MAX_LEVEL) {
						fprintf(stderr, "level: level is too high(input value less than %d).\n", MAX_LEVEL);
						exit(1);
					}
				} else {
					fprintf(stderr, "level: invalid start level(input non-zero value).\n");	
					exit(1);
				}
				
				break;
				
			case 's': /* -s or --server */
				server_mode = 1;
				break;

			case 'c': /* -c or --client */
				server_address = optarg;
				break;

			case 'p': /* -p or --port */
				port_number = optarg;
		
				if(is_number(port_number)) {
					port = atoi(port_number);		
				} else {
					fprintf(stderr, "port: invalid port number\n");
					exit(1);
				}
				
				break;
				
			case '?': /* The user specified an invalid option. */
				print_usage(stderr, 1);
				break;

			case -1: /* Done with options. */
				break;

			default: /* Something else: unexpected. */
				abort();
		}
	} while(next_option != -1);
	
	if(server_address != NULL && server_mode == 1) {
		print_usage(stderr, 1);
	} else if(server_mode) {
		if(port_number == NULL) 
			port = DEFAULT_PORT;

		if(start_level == NULL)
			level = DEFAULT_LEVEL;
		
		server_mode_direct(port, level);
	} else if(server_address != NULL) {
		if(start_level != NULL)
			print_usage(stderr, 1); 
		
		if(port_number == NULL) 
			port = DEFAULT_PORT;
		
		client_mode_direct(server_address, port);
	} else {
		if(port_number != NULL) {
			print_usage(stderr, 1);
		} else if(start_level != NULL) {
			start_single_game_direct(level);
		} else {
			set_signal_handler();
			initscr();
			curs_set(1);
			echo();
			cbreak();	
			keypad(stdscr, TRUE);	
			srand(time(NULL));

			show_main_menu();
		}
	}
	
	endwin();
	
	return 0;
}

void print_usage(FILE *stream, int exit_code) {
	fprintf(stream, "Usage: %s [{{-s | -c <server address>} [-p <port number>] | -l <start level>}]\n", program_name);
	fprintf(stream,
			"       -h      --help                      Display this usage information.\n"
			"       -l      --level <start level>       Run as single play mode and specify the start level.\n"
			"       -c      --client <server address>   Run as a client mode and specify the server address.\n"
			"       -s      --server                    Run as a server mode.\n"
			"       -p      --port <port number>        Specify the port number(default: %d).\n", DEFAULT_LEVEL, MAX_LEVEL, DEFAULT_PORT);
	
	exit(exit_code);
}
